@@ -1,12 +1,12 @@
-#!/usr/bin/python
-# Copyright 2009 Gentoo Foundation
+#!/usr/bin/python -b
+# Copyright 2009-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: xpak-helper.py 14576 2009-10-11 22:51:08Z zmedico $
 
-import optparse
 import sys
 import portage
+portage._internal_caller = True
 from portage import os
+from portage.util._argparse import ArgumentParser
 
 def command_recompose(args):
 
@@ -37,7 +37,7 @@ def command_recompose(args):
 
 def main(argv):
 
-	if argv and sys.hexversion < 0x3000000 and not isinstance(argv[0], unicode):
+	if argv and isinstance(argv[0], bytes):
 		for i, x in enumerate(argv):
 			argv[i] = portage._unicode_decode(x, errors='strict')
 
@@ -46,8 +46,8 @@ def main(argv):
 	usage = "usage: %s COMMAND [args]" % \
 		os.path.basename(argv[0])
 
-	parser = optparse.OptionParser(description=description, usage=usage)
-	options, args = parser.parse_args(argv[1:])
+	parser = ArgumentParser(description=description, usage=usage)
+	options, args = parser.parse_known_args(argv[1:])
 
 	if not args:
 		parser.error("missing command argument")
