@@ -1,4 +1,4 @@
-# Copyright 1999-2012 Gentoo Foundation
+# Copyright 1999-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from _emerge.AsynchronousLock import AsynchronousLock
@@ -24,7 +24,8 @@ class BinpkgFetcher(SpawnProcess):
 	def __init__(self, **kwargs):
 		SpawnProcess.__init__(self, **kwargs)
 		pkg = self.pkg
-		self.pkg_path = pkg.root_config.trees["bintree"].getname(pkg.cpv)
+		self.pkg_path = pkg.root_config.trees["bintree"].getname(
+			pkg.cpv) + ".partial"
 
 	def _start(self):
 
@@ -51,10 +52,12 @@ class BinpkgFetcher(SpawnProcess):
 		# urljoin doesn't work correctly with
 		# unrecognized protocols like sftp
 		if bintree._remote_has_index:
-			rel_uri = bintree._remotepkgs[pkg.cpv].get("PATH")
+			instance_key = bintree.dbapi._instance_key(pkg.cpv)
+			rel_uri = bintree._remotepkgs[instance_key].get("PATH")
 			if not rel_uri:
 				rel_uri = pkg.cpv + ".tbz2"
-			remote_base_uri = bintree._remotepkgs[pkg.cpv]["BASE_URI"]
+			remote_base_uri = bintree._remotepkgs[
+				instance_key]["BASE_URI"]
 			uri = remote_base_uri.rstrip("/") + "/" + rel_uri.lstrip("/")
 		else:
 			uri = settings["PORTAGE_BINHOST"].rstrip("/") + \
@@ -63,7 +66,7 @@ class BinpkgFetcher(SpawnProcess):
 		if pretend:
 			portage.writemsg_stdout("\n%s\n" % uri, noiselevel=-1)
 			self._set_returncode((self.pid, os.EX_OK << 8))
-			self.wait()
+			self._async_wait()
 			return
 
 		protocol = urllib_parse_urlparse(uri)[0]
@@ -80,6 +83,12 @@ class BinpkgFetcher(SpawnProcess):
 			"FILE"    : os.path.basename(pkg_path)
 		}
 
+		for k in ("PORTAGE_SSH_OPTS",):
+			try:
+				fcmd_vars[k] = settings[k]
+			except KeyError:
+				pass
+
 		fetch_env = dict(settings.items())
 		fetch_args = [portage.util.varexpand(x, mydict=fcmd_vars) \
 			for x in portage.util.shlex_split(fcmd)]
@@ -122,7 +131,9 @@ class BinpkgFetcher(SpawnProcess):
 			# the fetcher didn't already do it automatically.
 			bintree = self.pkg.root_config.trees["bintree"]
 			if bintree._remote_has_index:
-				remote_mtime = bintree._remotepkgs[self.pkg.cpv].get("MTIME")
+				remote_mtime = bintree._remotepkgs[
+					bintree.dbapi._instance_key(
+					self.pkg.cpv)].get("MTIME")
 				if remote_mtime is not None:
 					try:
 						remote_mtime = long(remote_mtime)
