@@ -1,5 +1,5 @@
 # data.py -- Calculated/Discovered Data Values
-# Copyright 1998-2013 Gentoo Foundation
+# Copyright 1998-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 import os, pwd, grp, platform, sys
@@ -8,11 +8,12 @@ import portage
 portage.proxy.lazyimport.lazyimport(globals(),
 	'portage.output:colorize',
 	'portage.util:writemsg',
+	'portage.util.path:first_existing',
 	'subprocess'
 )
 from portage.localization import _
 
-ostype=platform.system()
+ostype = platform.system()
 userland = None
 if ostype == "DragonFly" or ostype.endswith("BSD"):
 	userland = "BSD"
@@ -23,10 +24,10 @@ lchown = getattr(os, "lchown", None)
 
 if not lchown:
 	if ostype == "Darwin":
-		def lchown(*pos_args, **key_args):
+		def lchown(*_args, **_kwargs):
 			pass
 	else:
-		def lchown(*pargs, **kwargs):
+		def lchown(*_args, **_kwargs):
 			writemsg(colorize("BAD", "!!!") + _(
 				" It seems that os.lchown does not"
 				" exist.  Please rebuild python.\n"), noiselevel=-1)
@@ -34,6 +35,35 @@ if not lchown:
 
 lchown = portage._unicode_func_wrapper(lchown)
 
+def _target_eprefix():
+	"""
+	Calculate the target EPREFIX, which may be different from
+	portage.const.EPREFIX due to cross-prefix support. The result
+	is equivalent to portage.settings["EPREFIX"], but the calculation
+	is done without the expense of instantiating portage.settings.
+	@rtype: str
+	@return: the target EPREFIX
+	"""
+	eprefix = os.environ.get("EPREFIX", portage.const.EPREFIX)
+	if eprefix:
+		eprefix = portage.util.normalize_path(eprefix)
+	return eprefix
+
+def _target_root():
+	"""
+	Calculate the target ROOT. The result is equivalent to
+	portage.settings["ROOT"], but the calculation
+	is done without the expense of instantiating portage.settings.
+	@rtype: str
+	@return: the target ROOT (always ends with a slash)
+	"""
+	root = os.environ.get("ROOT")
+	if not root:
+		# Handle either empty or unset ROOT.
+		root = os.sep
+	root = portage.util.normalize_path(root)
+	return root.rstrip(os.sep) + os.sep
+
 def portage_group_warning():
 	warn_prefix = colorize("BAD", "*** WARNING ***  ")
 	mylines = [
@@ -59,11 +89,19 @@ def portage_group_warning():
 # If the "wheel" group does not exist then wheelgid falls back to 0.
 # If the "portage" group does not exist then portage_uid falls back to wheelgid.
 
-uid=os.getuid()
-wheelgid=0
+# If the current user is not root, but has write access to the
+# EROOT directory (not due to the 0002 bit), then use "unprivileged"
+# mode which sets secpass = 2 and uses the UID and GID of the EROOT
+# directory to generate default PORTAGE_INST_GID, PORTAGE_INST_UID,
+# PORTAGE_USERNAME, and PORTAGE_GRPNAME settings.
+def _unprivileged_mode(eroot, eroot_st):
+	return os.getuid() != 0 and os.access(eroot, os.W_OK) and \
+		not eroot_st.st_mode & 0o0002
 
+uid = os.getuid()
+wheelgid = 0
 try:
-	wheelgid=grp.getgrnam("wheel")[2]
+	wheelgid = grp.getgrnam("wheel")[2]
 except KeyError:
 	pass
 
@@ -78,13 +116,34 @@ def _get_global(k):
 	if k in _initialized_globals:
 		return globals()[k]
 
-	if k in ('portage_gid', 'portage_uid', 'secpass'):
-		global portage_gid, portage_uid, secpass
-		secpass = 0
+	if k == 'secpass':
+
+		unprivileged = False
+		if hasattr(portage, 'settings'):
+			unprivileged = "unprivileged" in portage.settings.features
+		else:
+			# The config class has equivalent code, but we also need to
+			# do it here if _disable_legacy_globals() has been called.
+			eroot_or_parent = first_existing(os.path.join(
+				_target_root(), _target_eprefix().lstrip(os.sep)))
+			try:
+				eroot_st = os.stat(eroot_or_parent)
+			except OSError:
+				pass
+			else:
+				unprivileged = _unprivileged_mode(
+					eroot_or_parent, eroot_st)
+
+		v = 0
 		if uid == 0:
-			secpass = 2
-		elif portage.const.EPREFIX:
-			secpass = 2
+			v = 2
+		elif unprivileged:
+			v = 2
+		elif portage_gid in os.getgroups():
+			v = 1
+
+	elif k in ('portage_gid', 'portage_uid'):
+
 		#Discover the uid and gid of the portage user/group
 		keyerror = False
 		try:
@@ -99,9 +158,6 @@ def _get_global(k):
 			keyerror = True
 			portage_gid = 0
 
-		if secpass < 1 and portage_gid in os.getgroups():
-			secpass = 1
-
 		# Suppress this error message if both PORTAGE_GRPNAME and
 		# PORTAGE_USERNAME are set to "root", for things like
 		# Android (see bug #454060).
@@ -119,21 +175,20 @@ def _get_global(k):
 				noiselevel=-1)
 			portage_group_warning()
 
+		globals()['portage_gid'] = portage_gid
 		_initialized_globals.add('portage_gid')
+		globals()['portage_uid'] = portage_uid
 		_initialized_globals.add('portage_uid')
-		_initialized_globals.add('secpass')
 
 		if k == 'portage_gid':
 			return portage_gid
 		elif k == 'portage_uid':
 			return portage_uid
-		elif k == 'secpass':
-			return secpass
 		else:
 			raise AssertionError('unknown name: %s' % k)
 
 	elif k == 'userpriv_groups':
-		v = [portage_gid]
+		v = [_get_global('portage_gid')]
 		if secpass >= 2:
 			# Get a list of group IDs for the portage user. Do not use
 			# grp.getgrall() since it is known to trigger spurious
@@ -179,32 +234,31 @@ def _get_global(k):
 			v = os.environ[env_key]
 		elif hasattr(portage, 'settings'):
 			v = portage.settings.get(env_key)
-		elif portage.const.EPREFIX:
-			# For prefix environments, default to the UID and GID of
-			# the top-level EROOT directory. The config class has
-			# equivalent code, but we also need to do it here if
-			# _disable_legacy_globals() has been called.
-			eroot = os.path.join(os.environ.get('ROOT', os.sep),
-				portage.const.EPREFIX.lstrip(os.sep))
+		else:
+			# The config class has equivalent code, but we also need to
+			# do it here if _disable_legacy_globals() has been called.
+			eroot_or_parent = first_existing(os.path.join(
+				_target_root(), _target_eprefix().lstrip(os.sep)))
 			try:
-				eroot_st = os.stat(eroot)
+				eroot_st = os.stat(eroot_or_parent)
 			except OSError:
 				pass
 			else:
-				if k == '_portage_grpname':
-					try:
-						grp_struct = grp.getgrgid(eroot_st.st_gid)
-					except KeyError:
-						pass
+				if _unprivileged_mode(eroot_or_parent, eroot_st):
+					if k == '_portage_grpname':
+						try:
+							grp_struct = grp.getgrgid(eroot_st.st_gid)
+						except KeyError:
+							pass
+						else:
+							v = grp_struct.gr_name
 					else:
-						v = grp_struct.gr_name
-				else:
-					try:
-						pwd_struct = pwd.getpwuid(eroot_st.st_uid)
-					except KeyError:
-						pass
-					else:
-						v = pwd_struct.pw_name
+						try:
+							pwd_struct = pwd.getpwuid(eroot_st.st_uid)
+						except KeyError:
+							pass
+						else:
+							v = pwd_struct.pw_name
 
 		if v is None:
 			v = 'portage'
@@ -255,3 +309,14 @@ def _init(settings):
 			v = portage._native_string(v)
 		globals()['_portage_username'] = v
 		_initialized_globals.add('_portage_username')
+
+	if 'secpass' not in _initialized_globals:
+		v = 0
+		if uid == 0:
+			v = 2
+		elif "unprivileged" in settings.features:
+			v = 2
+		elif portage_gid in os.getgroups():
+			v = 1
+		globals()['secpass'] = v
+		_initialized_globals.add('secpass')
