@@ -1,31 +1,40 @@
-# Copyright 2010 Gentoo Foundation
+# Copyright 2010-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: prepare_build_dirs.py 15657 2010-03-02 20:59:00Z zmedico $
+
+from __future__ import unicode_literals
 
 __all__ = ['prepare_build_dirs']
 
-import codecs
 import errno
-import shutil
+import gzip
 import stat
 import time
 
-from portage import os, _encodings, _unicode_encode, _unicode_decode
+import portage
+from portage import os, shutil, _encodings, _unicode_encode, _unicode_decode
 from portage.data import portage_gid, portage_uid, secpass
 from portage.exception import DirectoryNotFound, FileNotFound, \
 	OperationNotPermitted, PermissionDenied, PortageException
 from portage.localization import _
 from portage.output import colorize
 from portage.util import apply_recursive_permissions, \
-	apply_secpass_permissions, ensure_dirs, writemsg
+	apply_secpass_permissions, ensure_dirs, normalize_path, writemsg
 
-def prepare_build_dirs(myroot, mysettings, cleanup):
+def prepare_build_dirs(myroot=None, settings=None, cleanup=False):
+	"""
+	The myroot parameter is ignored.
+	"""
+	myroot = None
 
+	if settings is None:
+		raise TypeError("settings argument is required")
+
+	mysettings = settings
 	clean_dirs = [mysettings["HOME"]]
 
 	# We enable cleanup when we want to make sure old cruft (such as the old
 	# environment) doesn't interfere with the current phase.
-	if cleanup:
+	if cleanup and 'keeptemp' not in mysettings.features:
 		clean_dirs.append(mysettings["T"])
 
 	for clean_dir in clean_dirs:
@@ -65,15 +74,14 @@ def prepare_build_dirs(myroot, mysettings, cleanup):
 	try:
 		for mydir in mydirs:
 			ensure_dirs(mydir)
-			apply_secpass_permissions(mydir,
-				gid=portage_gid, uid=portage_uid, mode=0o70, mask=0)
+			try:
+				apply_secpass_permissions(mydir,
+					gid=portage_gid, uid=portage_uid, mode=0o700, mask=0)
+			except PortageException:
+				if not os.path.isdir(mydir):
+					raise
 		for dir_key in ("PORTAGE_BUILDDIR", "HOME", "PKG_LOGDIR", "T"):
-			"""These directories don't necessarily need to be group writable.
-			However, the setup phase is commonly run as a privileged user prior
-			to the other phases being run by an unprivileged user.  Currently,
-			we use the portage group to ensure that the unprivleged user still
-			has write access to these directories in any case."""
-			ensure_dirs(mysettings[dir_key], mode=0o775)
+			ensure_dirs(mysettings[dir_key], mode=0o755)
 			apply_secpass_permissions(mysettings[dir_key],
 				uid=portage_uid, gid=portage_gid)
 	except PermissionDenied as e:
@@ -94,7 +102,7 @@ def prepare_build_dirs(myroot, mysettings, cleanup):
 			pass
 
 	_prepare_workdir(mysettings)
-	if mysettings.get('EBUILD_PHASE') != 'fetch':
+	if mysettings.get("EBUILD_PHASE") not in ("info", "fetch", "pretend"):
 		# Avoid spurious permissions adjustments when fetching with
 		# a temporary PORTAGE_TMPDIR setting (for fetchonly).
 		_prepare_features_dirs(mysettings)
@@ -107,18 +115,22 @@ def _adjust_perms_msg(settings, msg):
 	background = settings.get("PORTAGE_BACKGROUND") == "1"
 	log_path = settings.get("PORTAGE_LOG_FILE")
 	log_file = None
+	log_file_real = None
 
 	if background and log_path is not None:
 		try:
-			log_file = codecs.open(_unicode_encode(log_path,
-				encoding=_encodings['fs'], errors='strict'),
-				mode='a', encoding=_encodings['content'], errors='replace')
+			log_file = open(_unicode_encode(log_path,
+				encoding=_encodings['fs'], errors='strict'), mode='ab')
+			log_file_real = log_file
 		except IOError:
 			def write(msg):
 				pass
 		else:
+			if log_path.endswith('.gz'):
+				log_file =  gzip.GzipFile(filename='',
+					mode='ab', fileobj=log_file)
 			def write(msg):
-				log_file.write(_unicode_decode(msg))
+				log_file.write(_unicode_encode(msg))
 				log_file.flush()
 
 	try:
@@ -126,17 +138,27 @@ def _adjust_perms_msg(settings, msg):
 	finally:
 		if log_file is not None:
 			log_file.close()
+			if log_file_real is not log_file:
+				log_file_real.close()
 
 def _prepare_features_dirs(mysettings):
 
+	# Use default ABI libdir in accordance with bug #355283.
+	libdir = None
+	default_abi = mysettings.get("DEFAULT_ABI")
+	if default_abi:
+		libdir = mysettings.get("LIBDIR_" + default_abi)
+	if not libdir:
+		libdir = "lib"
+
 	features_dirs = {
 		"ccache":{
-			"path_dir": "/usr/lib/ccache/bin",
+			"path_dir": "/usr/%s/ccache/bin" % (libdir,),
 			"basedir_var":"CCACHE_DIR",
 			"default_dir":os.path.join(mysettings["PORTAGE_TMPDIR"], "ccache"),
 			"always_recurse":False},
 		"distcc":{
-			"path_dir": "/usr/lib/distcc/bin",
+			"path_dir": "/usr/%s/distcc/bin" % (libdir,),
 			"basedir_var":"DISTCC_DIR",
 			"default_dir":os.path.join(mysettings["BUILD_PREFIX"], ".distcc"),
 			"subdirs":("lock", "state"),
@@ -228,7 +250,6 @@ def _prepare_features_dirs(mysettings):
 
 			if failure:
 				mysettings.features.remove(myfeature)
-				mysettings['FEATURES'] = ' '.join(sorted(mysettings.features))
 				time.sleep(5)
 
 def _prepare_workdir(mysettings):
@@ -266,6 +287,11 @@ def _prepare_workdir(mysettings):
 		try:
 			modified = ensure_dirs(mysettings["PORT_LOGDIR"])
 			if modified:
+				# Only initialize group/mode if the directory doesn't
+				# exist, so that we don't override permissions if they
+				# were previously set by the administrator.
+				# NOTE: These permissions should be compatible with our
+				# default logrotate config as discussed in bug 374287.
 				apply_secpass_permissions(mysettings["PORT_LOGDIR"],
 					uid=portage_uid, gid=portage_gid, mode=0o2770)
 		except PortageException as e:
@@ -275,30 +301,103 @@ def _prepare_workdir(mysettings):
 			writemsg(_("!!! Disabling logging.\n"), noiselevel=-1)
 			while "PORT_LOGDIR" in mysettings:
 				del mysettings["PORT_LOGDIR"]
+
+	compress_log_ext = ''
+	if 'compress-build-logs' in mysettings.features:
+		compress_log_ext = '.gz'
+
+	logdir_subdir_ok = False
 	if "PORT_LOGDIR" in mysettings and \
 		os.access(mysettings["PORT_LOGDIR"], os.W_OK):
+		logdir = normalize_path(mysettings["PORT_LOGDIR"])
 		logid_path = os.path.join(mysettings["PORTAGE_BUILDDIR"], ".logid")
 		if not os.path.exists(logid_path):
-			open(_unicode_encode(logid_path), 'w')
+			open(_unicode_encode(logid_path), 'w').close()
 		logid_time = _unicode_decode(time.strftime("%Y%m%d-%H%M%S",
 			time.gmtime(os.stat(logid_path).st_mtime)),
 			encoding=_encodings['content'], errors='replace')
 
 		if "split-log" in mysettings.features:
+			log_subdir = os.path.join(logdir, "build", mysettings["CATEGORY"])
 			mysettings["PORTAGE_LOG_FILE"] = os.path.join(
-				mysettings["PORT_LOGDIR"], "build", "%s/%s:%s.log" % \
-				(mysettings["CATEGORY"], mysettings["PF"], logid_time))
+				log_subdir, "%s:%s.log%s" %
+				(mysettings["PF"], logid_time, compress_log_ext))
 		else:
+			log_subdir = logdir
 			mysettings["PORTAGE_LOG_FILE"] = os.path.join(
-				mysettings["PORT_LOGDIR"], "%s:%s:%s.log" % \
-				(mysettings["CATEGORY"], mysettings["PF"], logid_time))
+				logdir, "%s:%s:%s.log%s" % \
+				(mysettings["CATEGORY"], mysettings["PF"], logid_time,
+				compress_log_ext))
 
-		ensure_dirs(os.path.dirname(mysettings["PORTAGE_LOG_FILE"]))
+		if log_subdir is logdir:
+			logdir_subdir_ok = True
+		else:
+			try:
+				_ensure_log_subdirs(logdir, log_subdir)
+			except PortageException as e:
+				writemsg("!!! %s\n" % (e,), noiselevel=-1)
 
-	else:
+			if os.access(log_subdir, os.W_OK):
+				logdir_subdir_ok = True
+			else:
+				writemsg("!!! %s: %s\n" %
+					(_("Permission Denied"), log_subdir), noiselevel=-1)
+
+	tmpdir_log_path = os.path.join(
+		mysettings["T"], "build.log%s" % compress_log_ext)
+	if not logdir_subdir_ok:
 		# NOTE: When sesandbox is enabled, the local SELinux security policies
 		# may not allow output to be piped out of the sesandbox domain. The
 		# current policy will allow it to work when a pty is available, but
 		# not through a normal pipe. See bug #162404.
-		mysettings["PORTAGE_LOG_FILE"] = os.path.join(
-			mysettings["T"], "build.log")
+		mysettings["PORTAGE_LOG_FILE"] = tmpdir_log_path
+	else:
+		# Create a symlink from tmpdir_log_path to PORTAGE_LOG_FILE, as
+		# requested in bug #412865.
+		make_new_symlink = False
+		try:
+			target = os.readlink(tmpdir_log_path)
+		except OSError:
+			make_new_symlink = True
+		else:
+			if target != mysettings["PORTAGE_LOG_FILE"]:
+				make_new_symlink = True
+		if make_new_symlink:
+			try:
+				os.unlink(tmpdir_log_path)
+			except OSError:
+				pass
+			os.symlink(mysettings["PORTAGE_LOG_FILE"], tmpdir_log_path)
+
+def _ensure_log_subdirs(logdir, subdir):
+	"""
+	This assumes that logdir exists, and creates subdirectories down
+	to subdir as necessary. The gid of logdir is copied to all
+	subdirectories, along with 0x2070 mode bits if present. Both logdir
+	and subdir are assumed to be normalized absolute paths.
+	"""
+	st = os.stat(logdir)
+	uid = -1
+	gid = st.st_gid
+	grp_mode = 0o2070 & st.st_mode
+
+	# If logdir is writable by the portage group but its uid
+	# is not portage_uid, then set the uid to portage_uid if
+	# we have privileges to do so, for compatibility with our
+	# default logrotate config (see bug 378451). With the
+	# "su portage portage" directive and logrotate-3.8.0,
+	# logrotate's chown call during the compression phase will
+	# only succeed if the log file's uid is portage_uid.
+	if grp_mode and gid == portage_gid and \
+		portage.data.secpass >= 2:
+		uid = portage_uid
+		if st.st_uid != portage_uid:
+			ensure_dirs(logdir, uid=uid)
+
+	logdir_split_len = len(logdir.split(os.sep))
+	subdir_split = subdir.split(os.sep)[logdir_split_len:]
+	subdir_split.reverse()
+	current = logdir
+	while subdir_split:
+		current = os.path.join(current, subdir_split.pop())
+		ensure_dirs(current, uid=uid, gid=gid, mode=grp_mode, mask=0)
