@@ -1,6 +1,5 @@
-# Copyright 2009 Gentoo Foundation
+# Copyright 2009-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: lazyimport.py 14327 2009-09-21 16:07:07Z arfrever $
 
 __all__ = ['lazyimport']
 
@@ -15,11 +14,35 @@ except ImportError:
 from portage.proxy.objectproxy import ObjectProxy
 
 if sys.hexversion >= 0x3000000:
+	# pylint: disable=W0622
 	basestring = str
 
 _module_proxies = {}
 _module_proxies_lock = threading.RLock()
 
+def _preload_portage_submodules():
+	"""
+	Load lazily referenced portage submodules into memory,
+	so imports won't fail during portage upgrade/downgrade.
+	Note that this recursively loads only the modules that
+	are lazily referenced by currently imported modules,
+	so some portage submodules may still remain unimported
+	after this function is called.
+	"""
+	imported = set()
+	while True:
+		remaining = False
+		for name in list(_module_proxies):
+			if name.startswith('portage.') or name.startswith('_emerge.'):
+				if name in imported:
+					continue
+				imported.add(name)
+				remaining = True
+				__import__(name)
+				_unregister_module_proxy(name)
+		if not remaining:
+			break
+
 def _register_module_proxy(name, proxy):
 	_module_proxies_lock.acquire()
 	try:
@@ -36,19 +59,13 @@ def _unregister_module_proxy(name):
 	Destroy all proxies that reference the give module name. Also, check
 	for other proxies referenced by modules that have been imported and
 	destroy those proxies too. This way, destruction of a single proxy
-	can trigger destruction of all the rest.
+	can trigger destruction of all the rest. If a target module appears
+	to be partially imported (indicated when an AttributeError is caught),
+	this function will leave in place proxies that reference it.
 	"""
 	_module_proxies_lock.acquire()
 	try:
-		proxy_list = _module_proxies.get(name)
-		if proxy_list is not None:
-			# First delete this name from the dict so that
-			# if this same thread reenters below, it won't
-			# enter this path again.
-			del _module_proxies[name]
-			for proxy in proxy_list:
-				object.__getattribute__(proxy, '_get_target')()
-
+		if name in _module_proxies:
 			modules = sys.modules
 			for name, proxy_list in list(_module_proxies.items()):
 				if name not in modules:
@@ -57,8 +74,16 @@ def _unregister_module_proxy(name):
 				# if this same thread reenters below, it won't
 				# enter this path again.
 				del _module_proxies[name]
-				for proxy in proxy_list:
-					object.__getattribute__(proxy, '_get_target')()
+				try:
+					while proxy_list:
+						proxy = proxy_list.pop()
+						object.__getattribute__(proxy, '_get_target')()
+				except AttributeError:
+					# Apparently the target module is only partially
+					# imported, so proxies that reference it cannot
+					# be destroyed yet.
+					proxy_list.append(proxy)
+					_module_proxies[name] = proxy_list
 	finally:
 		_module_proxies_lock.release()
 
@@ -103,6 +128,9 @@ class _LazyImportFrom(_LazyImport):
 		name = object.__getattribute__(self, '_name')
 		attr_name = object.__getattribute__(self, '_attr_name')
 		__import__(name)
+		# If called by _unregister_module_proxy() and the target module is
+		# partially imported, then the following getattr call may raise an
+		# AttributeError for _unregister_module_proxy() to handle.
 		target = getattr(sys.modules[name], attr_name)
 		object.__setattr__(self, '_target', target)
 		object.__getattribute__(self, '_scope')[
@@ -162,6 +190,9 @@ def lazyimport(scope, *args):
 			already_imported = modules.get(name)
 			fromlist = fromlist.split(',')
 			for s in fromlist:
+				if not s:
+					# This happens if there's an extra comma in fromlist.
+					raise ValueError('Empty module attribute name')
 				alias = s.split('@', 1)
 				if len(alias) == 1:
 					alias = alias[0]
@@ -172,7 +203,11 @@ def lazyimport(scope, *args):
 					try:
 						scope[alias] = getattr(already_imported, attr_name)
 					except AttributeError:
-						raise ImportError('cannot import name %s' % attr_name)
+						# Apparently the target module is only partially
+						# imported, so create a proxy.
+						already_imported = None
+						scope[alias] = \
+							_LazyImportFrom(scope, name, attr_name, alias)
 				else:
 					scope[alias] = \
 						_LazyImportFrom(scope, name, attr_name, alias)
